{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:  mwPasToHtml                               }
{           Purpose:          mwPasToHtml                               }
{           Source Language:  Delphi                                    }
{           Author Name and Contact Information:                        }
{           Modified by Greg Nixon <nzcs@xtra.co.nz>                    }
{           Martin Waldenburg's mwPasToRtf                              }
{           Copyright    1997-1998, all rights reserved.                }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Pas to Html converter for syntax highlighting etc.          }
{           FreeWare                                                    }
{           DISCLAIMER:  This is provided as is, expressly without a    }
{           warranty of any kind. You use it at your own risc.          }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Revision History                                            }
{           v#    ,Date       ,Author Name            ,Description      }
{           1.00  ,04/03/1998 ,Greg Nixon             ,Start            }
{           1.01  ,05/03/1998 ,Greg Nixon             ,Added Color      }
{           1.02  ,05/03/1998 ,Greg Nixon             ,Add NotDelphi    }
{           1.03  ,6/03/1998  ,Greg Nixon             ,Special Chars    }
{                                                                       }
{-----------------------------------------------------------------------}

unit mwPasToHtml;


interface

uses
   Windows, SysUtils, Messages, Classes, ComCtrls, Graphics, Dialogs,
   Registry;
   
type
   TTokenState = (tsAssembler, tsComment, tsCRLF, tsDirective, tsIdentifier,
                  tsKeyWord, tsNumber, tsSpace, tsString, tsSymbol, tsUnknown);
   
   TCommentState = (csAnsi, csBor, csNo, csSlashes);
   
type
   TPasConversion = class(TMemoryStream)
   private
      FTitle, FHeader: String;
      FPlainText: TColor;
      FBackGround: TColor;
      FNotDelphi: Boolean;
      FDiffer: Boolean;
      FPreFixList, FPostFixList: array[tsAssembler..tsUnknown] of String;
      FComment: TCommentState;
      Prefix, TokenStr, Postfix: String;
      FBuffPos, TokenLen, FOutBuffSize, FStrBuffSize: Integer;
      FReadBuff, FOutBuff, FStrBuff, FStrBuffEnd, Run, RunStr, TokenPtr: PChar;
      FTokenState: TTokenState;
      FAssemblerFo: TFont;
      FCommentFo: TFont;
      FDirectiveFo: TFont;
      FIdentifierFo: TFont;
      FNumberFo: TFont;
      FKeyWordFo: TFont;
      FSpaceFo: TFont;
      FStringFo: TFont;
      FSymbolFo: TFont;
      function IsKeyWord(aToken: String): Boolean;
      function IsDirective(aToken: String): Boolean;
      function IsDiffKey(aToken: String): Boolean;
      procedure SetAssemblerFo(newValue: TFont);
      procedure SetCommentFo(newValue: TFont);
      procedure SetDirectiveFo(newValue: TFont);
      procedure SetIdentifierFo(newValue: TFont);
      procedure SetKeyWordFo(newValue: TFont);
      procedure SetNumberFo(newValue: TFont);
      procedure SetSpaceFo(newValue: TFont);
      procedure SetStringFo(newValue: TFont);
      procedure SetSymbolFo(newValue: TFont);
      procedure SetHTML;
      procedure WriteToBuffer(aString: String);
      procedure HandleAnsiC;
      procedure HandleBorC;
      procedure HandleCRLF;
      procedure HandleSlashesC;
      procedure HandleString;
      procedure AllocStrBuff;
      procedure SetPreAndPosFix(aFont: TFont; aTokenState: TTokenState);
      function StrToHTML(const AString: String): String;
      function HTMLColor(Color: TColor): String;
      procedure UseDelphiHighlighting;
   protected
   public
      constructor Create;
      destructor Destroy; override;
      procedure Init;
      function ConvertReadStream: Integer;
      property AssemblerFo: TFont read FAssemblerFo write SetAssemblerFo;
      property CommentFo: TFont read FCommentFo write SetCommentFo;
      property DirectiveFo: TFont read FDirectiveFo write SetDirectiveFo;
      property IdentifierFo: TFont read FIdentifierFo write SetIdentifierFo;
      property KeyWordFo: TFont read FKeyWordFo write SetKeyWordFo;
      property NumberFo: TFont read FNumberFo write SetNumberFo;
      property SpaceFo: TFont read FSpaceFo write SetSpaceFo;
      property StringFo: TFont read FStringFo write SetStringFo;
      property SymbolFo: TFont read FSymbolFo write SetSymbolFo;
      // New Header and Title Properties //
      property Title: String read FTitle write FTitle;
      property Header: String read FHeader write FHeader;
      property BackGround: TColor read FBackGround write FBackGround;
      // Set to true to not highlight //
      property NotDelphi: Boolean read FNotDelphi write FNotDelphi Default False;
   published
   end;
      
const
   Keywords: array[0..98] of string =
   ('ABSOLUTE', 'ABSTRACT', 'AND', 'ARRAY', 'AS', 'ASM', 'ASSEMBLER',
   'AUTOMATED', 'BEGIN', 'CASE', 'CDECL', 'CLASS', 'CONST', 'CONSTRUCTOR',
   'DEFAULT', 'DESTRUCTOR', 'DISPID', 'DISPINTERFACE', 'DIV', 'DO',
   'DOWNTO', 'DYNAMIC', 'ELSE', 'END', 'EXCEPT', 'EXPORT', 'EXPORTS',
   'EXTERNAL', 'FAR', 'FILE', 'FINALIZATION', 'FINALLY', 'FOR', 'FORWARD',
   'FUNCTION', 'GOTO', 'IF', 'IMPLEMENTATION', 'IN', 'INDEX', 'INHERITED',
   'INITIALIZATION', 'INLINE', 'INTERFACE', 'IS', 'LABEL', 'LIBRARY',
   'MESSAGE', 'MOD', 'NAME', 'NEAR', 'NIL', 'NODEFAULT', 'NOT', 'OBJECT',
   'OF', 'OR', 'OUT', 'OVERRIDE', 'PACKED', 'PASCAL', 'PRIVATE', 'PROCEDURE',
   'PROGRAM', 'PROPERTY', 'PROTECTED', 'PUBLIC', 'PUBLISHED', 'RAISE',
   'READ', 'READONLY', 'RECORD', 'REGISTER', 'REPEAT', 'RESIDENT',
   'RESOURCESTRING', 'SAFECALL', 'SET', 'SHL', 'SHR', 'STDCALL', 'STORED',
   'STRING', 'STRINGRESOURCE', 'THEN', 'THREADVAR', 'TO', 'TRY', 'TYPE',
   'UNIT', 'UNTIL', 'USES', 'VAR', 'VIRTUAL', 'WHILE', 'WITH', 'WRITE',
   'WRITEONLY', 'XOR');
   
   Directives: array[0..10] of string =
   ('AUTOMATED', 'INDEX', 'NAME', 'NODEFAULT', 'READ', 'READONLY',
   'RESIDENT', 'STORED', 'STRINGRECOURCE', 'WRITE', 'WRITEONLY');
   
   DiffKeys: array[0..6] of string =
   ('END', 'FUNCTION', 'PRIVATE', 'PROCEDURE', 'PRODECTED', 'PUBLIC', 'PUBLISHED');
   

implementation

destructor TPasConversion.Destroy;
begin
   FAssemblerFo.Free;
   FCommentFo.Free;
   FDirectiveFo.Free;
   FIdentifierFo.Free;
   FKeyWordFo.Free;
   FNumberFo.Free;
   FSpaceFo.Free;
   FStringFo.Free;
   FSymbolFo.Free;
   ReAllocMem(FStrBuff, 0);
   inherited Destroy;
end;  { Destroy }

constructor TPasConversion.Create;
begin
   inherited Create;
   // Set Defaults //
   FTitle := 'Delphi Example';
   FHeader := 'Delphi Example Code';
   FPlainText := clBlack;
   FBackGround := clWhite;
   FNotDelphi := False;
   FAssemblerFo := TFont.Create;
   FCommentFo := TFont.Create;
   FDirectiveFo := TFont.Create;
   FIdentifierFo := TFont.Create;
   FKeyWordFo := TFont.Create;
   FNumberFo := TFont.Create;
   FSpaceFo := TFont.Create;
   FStringFo := TFont.Create;
   FSymbolFo := TFont.Create;
   Prefix := '';
   PostFix := '';
   FStrBuffSize := 0;
   AllocStrBuff;
   Init;
   FDiffer := False;
end;  { Create }

procedure TPasConversion.AllocStrBuff;
begin
   FStrBuffSize := FStrBuffSize + 1024;
   ReAllocMem(FStrBuff, FStrBuffSize);
   FStrBuffEnd := FStrBuff + 1023;
end;  { AllocStrBuff }

procedure TPasConversion.SetAssemblerFo(newValue: TFont);
begin
   FAssemblerFo.Assign(newValue);
   SetPreAndPosFix(newValue, tsAssembler);
end;  { SetAssemblerFo }

procedure TPasConversion.SetCommentFo(newValue: TFont);
begin
   FCommentFo.Assign(newValue);
   SetPreAndPosFix(newValue, tsComment);
end;  { SetCommentFo }

procedure TPasConversion.SetDirectiveFo(newValue: TFont);
begin
   FDirectiveFo.Assign(newValue);
   SetPreAndPosFix(newValue, tsDirective);
end;  { SetDirectiveFo }

procedure TPasConversion.SetIdentifierFo(newValue: TFont);
begin
   FIdentifierFo.Assign(newValue);
   SetPreAndPosFix(newValue, tsIdentifier);
end;  { SetIdentifierFo }

procedure TPasConversion.SetKeyWordFo(newValue: TFont);
begin
   FKeyWordFo.Assign(newValue);
   SetPreAndPosFix(newValue, tsKeyWord);
end;  { SetKeyWordFo }

procedure TPasConversion.SetNumberFo(newValue: TFont);
begin
   FNumberFo.Assign(newValue);
   SetPreAndPosFix(newValue, tsNumber);
end;  { SetNumberFo }

procedure TPasConversion.SetSpaceFo(newValue: TFont);
begin
   FSpaceFo.Assign(newValue);
   SetPreAndPosFix(newValue, tsSpace);
end;  { SetSpaceFo }

procedure TPasConversion.SetStringFo(newValue: TFont);
begin
   FStringFo.Assign(newValue);
   SetPreAndPosFix(newValue, tsString);
end;  { SetStringFo }

procedure TPasConversion.SetSymbolFo(newValue: TFont);
begin
   FSymbolFo.Assign(newValue);
   SetPreAndPosFix(newValue, tsSymbol);
end;  { SetSymbolFo }

function TPasConversion.HTMLColor(Color: TColor): String;
var
   Tmp: String;
begin
   Tmp := IntToHex(Longint(Color), 6);
   Result := '#' + Copy(tmp, 5, 2) + Copy(Tmp, 3, 2) + Copy(Tmp, 1, 2);
end; { HTMLColor }

function TPasConversion.StrToHTML(const AString: String): String;
var
  i : integer;
  SpecTag : string;
const
  SpecChars = ['<','>','&','"','',
               '','','','','','','','','','',
               '','','','','','','','','',''];

function SpecTags(AChar : Char) : string;
begin
   case AChar of
      '<': Result := '&LT;';
      '>': Result := '&GT;';
      '&': Result := '&AMP;';
      '"': Result := '&QUOT;';
      '': Result := '&COPY;';
      '': Result := '&EACUTE;';
      '': Result := '&eACUTE;';
      '': Result := '&EGRAVE;';
      '': Result := '&eGRAVE;';
      '': Result := '&ECIRC;';
      '': Result := '&eCIRC;';
      '': Result := '&EUML;';
      '': Result := '&eUML;';
      '': Result := '&IUML;';
      '': Result := '&iUML;';
      '': Result := '&ICIRC;';
      '': Result := '&iCIRC;';
      '': Result := '&CCEDIL;';
      '': Result := '&cCEDIL;';
      '': Result := '&AGRAVE;';
      '': Result := '&aGRAVE;';
      '': Result := '&UUML;';
      '': Result := '&uUML;';
      '': Result := '&UCIRC;';
      '': Result := '&uCIRC;';
   else
      Result := '_';
   end;
end;

begin
   Result := AString;
   for i := Length(AString) downto 1 do begin
      if AString[i] in SpecChars then begin
         SpecTag := SpecTags(AString[i]);
         Delete(Result,i,1);
         Insert(SpecTag,Result,i);
       end;
   end;
end; { StrToHTML }

procedure TPasConversion.UseDelphiHighlighting;
{------------------------------------------------------------------------}
{Delphi Editor settings are a comma delimited list of seven              }
{values as follows:                                                      }
{                                                                        }
{0 - Foreground color                                                    }
{1 - Background color                                                    }
{2 - font style                                                          }
{3 - Foreground Default                                                  }
{4 - Background Default                                                  }
{6 - Unknown                                                             }
{7 - Unknown                                                             }
{                                                                        }
{Currently this routine only handles setting the Bold, Italic, Underline }
{------------------------------------------------------------------------}

procedure SetDelphiHTML(S: String; aTokenState: TTokenState);
var
   Ed_List: TStringList;
   Font: TFont;
Begin
   Font := TFont.Create;
   Ed_List := TStringList.Create;
   Try
      Ed_List.CommaText := S;
      if pos('B', Ed_List[2])>0 then
         Font.Style := Font.Style + [fsBold];
      if pos('I', Ed_List[2])>0 then
         Font.Style := Font.Style + [fsItalic];
      if pos('U', Ed_List[2])>0 then
         Font.Style := Font.Style + [fsUnderLine];
      Font.Color := StrToIntDef(ed_list[0], clblack);
      If aTokenState = tsUnknown then
         FPlainText := StrToIntDef(ed_list[0], clblack);
      SetPreAndPosFix(Font, aTokenState);
   finally
      Ed_List.Free;
      Font.Free;
   End;
End;

const Delphi_Editor: array[0..10] of string = ('Assembler', 'Comment', 'IGNORE',
   'IGNORE', 'Identifier', 'Reserved_Word', 'Number', 'Whitespace', 'String',
   'Symbol', 'Plain_Text');
const Delphi_Defaults: array[0..10] of string = ('0,16777215,,0,1,0,15', '8388608,16777215,I,0,1,4,15',
   '255,65280,,0,0,9,10', '16777215,128,,0,0,15,1', '0,16777215,,0,1,0,15',
   '0,16777215,B,0,1,0,15', '0,16777215,,0,1,0,15', '0,16777215,,1,1,0,0',
   '32768,16777215,,0,1,2,0', '0,16777215,,0,1,0,15', '0,0,,0,1,0,0');
var
   RegIni: TRegIniFile;
   Ed_Setting, DelVer: String;
   i: Integer;
Begin
   If FNotDelphi then exit;
   // First see if they have D2 //
   RegIni := TRegIniFile.Create('Software\Borland\Delphi\2.0');
   DelVer := RegIni.ReadString('HighLight', Delphi_Editor[10], 'ERROR');
   If DelVer = 'ERROR' then begin
      RegIni.Free;
      // If they don't then use D3 - If they don't have this it will use the Defaults //
      RegIni := TRegIniFile.Create('Software\Borland\Delphi\3.0');
   end;

   Try
      for i := 0 to 10 do
         if Delphi_Editor[i]<>'IGNORE' then
      Begin
      Ed_Setting := RegIni.ReadString('HighLight', Delphi_Editor[i], Delphi_Defaults[i]);
      SetDelphiHTML(Ed_Setting, TTokenState(i));
      End;
   finally
      RegIni.Free;
   End;
End;

procedure TPasConversion.SetPreAndPosFix(aFont: TFont; aTokenState: TTokenState);
begin
   {-------------------------------------------}
   {Here you need to set the Pre - and PostFix }
   {accordingly to the aFont value             }
   {-------------------------------------------}
   FPreFixList[aTokenState] := '';
   FPostFixList[aTokenState] := '';
   If aFont.Color <> FPlainText then
      FPreFixList[aTokenState] := '<FONT COLOR="' + HTMLColor(aFont.Color) + '">';
   if (fsBold in aFont.Style) then begin
      FPreFixList[aTokenState] := FPreFixList[aTokenState] + '<B>';
      FPostFixList[aTokenState] := '</B>';
   end;
   if (fsItalic in aFont.Style) then begin
      FPreFixList[aTokenState] := FPreFixList[aTokenState] + '<I>';
      FPostFixList[aTokenState] := FPostFixList[aTokenState] + '</I>';
   end;
   if (fsUnderline in aFont.Style) then begin
      FPreFixList[aTokenState] := FPreFixList[aTokenState] + '<U>';
      FPostFixList[aTokenState] := FPostFixList[aTokenState] + '</U>';
   end;
   If aFont.Color <> FPlainText then
      FPostFixList[aTokenState] := FPostFixList[aTokenState] + '</FONT>';
end;  { SetPreAndPosFix }

procedure TPasConversion.HandleAnsiC;
begin
   while Run^ <> #0 do
   begin
      Case Run^ of
         #13:
         begin
            if TokenPtr <> Run then
            begin
               FTokenState := tsComment;
               TokenLen := Run - TokenPtr;
               SetString(TokenStr, TokenPtr, TokenLen);
               SetHTML;
               WriteToBuffer(Prefix + TokenStr + Postfix);
               TokenPtr := Run;
            end;
            HandleCRLF;
            dec(Run);
         end;

      '*': if (Run + 1 )^ = ')' then begin  inc(Run, 2); break; end;
      end;
      inc(Run);
   end;
   FTokenState := tsComment;
   TokenLen := Run - TokenPtr;
   SetString(TokenStr, TokenPtr, TokenLen);
   SetHTML;
   WriteToBuffer(Prefix + TokenStr + Postfix);
   TokenPtr := Run;
   FComment := csNo;
end;  { HandleAnsiC }

procedure TPasConversion.HandleBorC;
begin
   while Run^ <> #0 do
   begin
      Case Run^ of
         #13:
         begin
            if TokenPtr <> Run then
            begin
               FTokenState := tsComment;
               TokenLen := Run - TokenPtr;
               SetString(TokenStr, TokenPtr, TokenLen);
               SetHTML;
               WriteToBuffer(Prefix + TokenStr + Postfix);
               TokenPtr := Run;
            end;
            HandleCRLF;
            dec(Run);
         end;

      '}': begin  inc(Run); break; end;

      end;
      inc(Run);
   end;
   FTokenState := tsComment;
   TokenLen := Run - TokenPtr;
   SetString(TokenStr, TokenPtr, TokenLen);
   SetHTML;
   WriteToBuffer(Prefix + TokenStr + Postfix);
   TokenPtr := Run;
   FComment := csNo;
end;  { HandleBorC }

procedure TPasConversion.HandleCRLF;
begin
   if Run^ = #0 then exit;
   inc(Run, 2);
   FTokenState := tsCRLF;
   TokenLen := Run - TokenPtr;
   SetString(TokenStr, TokenPtr, TokenLen);
   SetHTML;
   WriteToBuffer(Prefix + TokenStr + Postfix);
   TokenPtr := Run;
   fComment := csNo;
   FTokenState := tsUnKnown;
   if Run^ = #13 then HandleCRLF;
end;  { HandleCRLF }

procedure TPasConversion.HandleSlashesC;
begin
   FTokenState := tsComment;
   while (Run^ <> #13) and (Run^ <> #0) do inc(Run);
   TokenLen := Run - TokenPtr;
   SetString(TokenStr, TokenPtr, TokenLen);
   SetHTML;
   WriteToBuffer(Prefix + TokenStr + Postfix);
   TokenPtr := Run;
   FComment := csNo;
end;  { HandleSlashesC }

procedure TPasConversion.HandleString;
begin
   FTokenState := tsSTring;
   FComment := csNo;
   repeat
      Case Run^ of
         #0, #10, #13: raise exception.Create('Invalid string');
      end;
      inc(Run);
   until Run^ = #39;
   inc(Run);
   TokenLen := Run - TokenPtr;
   SetString(TokenStr, TokenPtr, TokenLen);
   SetHTML;
   WriteToBuffer(Prefix + TokenStr + Postfix);
   TokenPtr := Run;
end;  { HandleString }

function TPasConversion.IsKeyWord(aToken: String): Boolean;
var
   First, Last, I, Compare: Integer;
   Token: String;
begin
   First := 0;
   Last := 98;
   Result := False;
   Token := UpperCase(aToken);
   while First <= Last do
   begin
      I := (First + Last) shr 1;
      Compare := CompareStr(Keywords[i], Token);
      if Compare = 0 then
      begin
         Result := True;
         break;
      end
      else
         if Compare < 0  then First := I + 1 else Last := I - 1;
   end;
end;  { IsKeyWord }

function TPasConversion.IsDiffKey(aToken: String): Boolean;
var
   First, Last, I, Compare: Integer;
   Token: String;
begin
   First := 0;
   Last := 6;
   Result := False;
   Token := UpperCase(aToken);
   while First <= Last do
   begin
      I := (First + Last) shr 1;
      Compare := CompareStr(DiffKeys[i], Token);
      if Compare = 0 then
      begin
         Result := True;
         break;
      end
      else
         if Compare < 0  then First := I + 1 else Last := I - 1;
   end;
end;  { IsDiffKey }

function TPasConversion.IsDirective(aToken: String): Boolean;
var
   First, Last, I, Compare: Integer;
   Token: String;
begin
   First := 0;
   Last := 10;
   Result := False;
   Token := UpperCase(aToken);
   if CompareStr('PROPERTY', Token) = 0 then FDiffer := True;
   if IsDiffKey(Token) then FDiffer := False;
   while First <= Last do
   begin
      I := (First + Last) shr 1;
      Compare := CompareStr(Directives[i], Token);
      if Compare = 0 then
      begin
         Result := True;
         if FDiffer then
         begin
            Result := False;
            if CompareStr('NAME', Token) = 0 then Result := True;
            if CompareStr('RESIDENT', Token) = 0 then Result := True;
            if CompareStr('STRINGRESOURCE', Token) = 0 then Result := True;
         end;
         break;
      end
      else
         if Compare < 0  then First := I + 1 else Last := I - 1;
   end;
end;  { IsDirective }

procedure TPasConversion.SetHTML;
begin
   prefix := FPreFixList[FTokenState];
   postfix := FPostFixList[FTokenState];
   Case FTokenState of
      tsAssembler: FTokenState := tsUnknown;
      tsComment: FTokenState := tsUnknown;

      tsCRLF:
      begin
         FTokenState := tsUnknown;
         FComment := csNo;
      end;

      tsDirective: FTokenState := tsUnknown;
      tsIdentifier: FTokenState := tsUnknown;
      tsNumber: FTokenState := tsUnknown;
      tsKeyWord: FTokenState := tsUnknown;
      tsSpace: FTokenState := tsUnknown;
      tsString: FTokenState := tsUnknown;
      tsSymbol: FTokenState := tsUnknown;
   end;
   TokenStr := StrToHTML(TokenStr);
end;  { SetHTML }

procedure TPasConversion.WriteToBuffer(aString: String);
var
   Count, Pos: Longint;
begin
   Count := Length(aString);
   if (FBuffPos >= 0) and (Count >= 0) then
   begin
      Pos := FBuffPos + Count;
      if Pos > 0 then
      begin
         if Pos >= FOutBuffSize then
         begin
            Try
               FOutBuffSize := FOutBuffSize + 16384;
               ReAllocMem(FOutBuff, FOutBuffSize);
            except
               raise exception.Create('conversions buffer to small');
            end;
         end;
         {System.Write(aString);}
         StrECopy((FOutBuff + FBuffPos), PChar(aString));
         FBuffPos := FBuffPos + Count;
         FOutBuff[FBuffPos] := #0;
      end;
   end;
end;  { WriteToBuffer }

function TPasConversion.ConvertReadStream: Integer;
const
   WRAP = #13#10;
begin
   UseDelphiHighlighting;
   FTokenState := tsUnknown;
   FOutBuffSize := size + 3;
   ReAllocMem(FOutBuff, FOutBuffSize);
   FComment := csNo;
   FBuffPos := 0;
   FReadBuff := Memory;
   {Write leading HTML}
   WriteToBuffer('<HTML>' + WRAP);
   WriteToBuffer('<HEAD>' + WRAP);
   WriteToBuffer(Format('<TITLE>%s</TITLE>', [FTitle]) + WRAP);
   WriteToBuffer('<META NAME="GENERATOR" CONTENT="Gregs Delphi Code Formatter (nzcs@xtra.co.nz)">' + wrap);
   WriteToBuffer('</HEAD>' + wrap);
   WriteToBuffer('<BODY TEXT="' + HTMLColor(FPlainText) + '" BGCOLOR="'
      + HTMLColor(FBackGround) + '">' + WRAP);
   WriteToBuffer(Format('<H1><CENTER>%s</CENTER></H1>', [FHeader]) + WRAP);
   WriteToBuffer('<A NAME="top"></A>' + WRAP);
   WriteToBuffer('<A HREF="#bottom">Jump to the end</A>' + WRAP);
   WriteToBuffer('<HR>' + WRAP);
   WriteToBuffer('<CODE>' + WRAP);
   WriteToBuffer('<PRE>' + WRAP + WRAP);

   Result := Read(FReadBuff^, Size);
   FReadBuff[Result] := #0;
   if Result > 0 then
   begin
      Run := FReadBuff;
      TokenPtr := Run;
      while Run^ <> #0 do
      begin
         Case Run^ of

            #13:
            begin
               FComment := csNo;
               HandleCRLF;
            end;

            #1..#9, #11, #12, #14..#32:
            begin
               while Run^ in [#1..#9, #11, #12, #14..#32] do inc(Run);
               FTokenState := tsSpace;
               TokenLen := Run - TokenPtr;
               SetString(TokenStr, TokenPtr, TokenLen);
               SetHTML;
               WriteToBuffer(Prefix + TokenStr + Postfix);
               TokenPtr := Run;
            end;

            'A'..'Z', 'a'..'z', '_':
            begin
               FTokenState := tsIdentifier;
               inc(Run);
               while Run^ in ['A'..'Z', 'a'..'z', '0'..'9', '_'] do inc(Run);
               TokenLen := Run - TokenPtr;
               SetString(TokenStr, TokenPtr, TokenLen);
               if IsKeyWord(TokenStr) then
               begin
                  if IsDirective(TokenStr) then FTokenState := tsDirective
                  else FTokenState := tsKeyWord;
               end;
               SetHTML;
               WriteToBuffer(Prefix + TokenStr + Postfix);
               TokenPtr := Run;
            end;

            '0'..'9':
            begin
               inc(Run);
               FTokenState := tsNumber;
               while Run^ in ['0'..'9', '.', 'e', 'E'] do inc(Run);
               TokenLen := Run - TokenPtr;
               SetString(TokenStr, TokenPtr, TokenLen);
               SetHTML;
               WriteToBuffer(Prefix + TokenStr + Postfix);
               TokenPtr := Run;
            end;

            '{':
            begin
               FComment := csBor;
               HandleBorC;
            end;

            '!', '"', '%', '&', '('..'/', ':'..'@', '['..'^', '`', '~':
            begin
               FTokenState := tsSymbol;
               while Run^ in ['!', '"', '%', '&', '('..'/', ':'..'@', '['..'^', '`', '~'] do
               begin
                  Case Run^ of
                     '/': if (Run + 1)^ = '/' then
                     begin
                        TokenLen := Run - TokenPtr;
                        SetString(TokenStr, TokenPtr, TokenLen);
                        SetHTML;
                        WriteToBuffer(Prefix + TokenStr + Postfix);
                        TokenPtr := Run;
                        FComment := csSlashes;
                        HandleSlashesC;
                        break;
                     end;

                     '(': if (Run + 1)^ = '*' then
                     begin
                        TokenLen := Run - TokenPtr;
                        SetString(TokenStr, TokenPtr, TokenLen);
                        SetHTML;
                        WriteToBuffer(Prefix + TokenStr + Postfix);
                        TokenPtr := Run;
                        FComment := csAnsi;
                        HandleAnsiC;
                        break;
                     end;
                  end;
                  inc(Run);
               end;
               TokenLen := Run - TokenPtr;
               SetString(TokenStr, TokenPtr, TokenLen);
               SetHTML;
               WriteToBuffer(Prefix + TokenStr + Postfix);
               TokenPtr := Run;
            end;

            #39: HandleString;

            '#':
            begin
               FTokenState := tsString;
               while Run^ in ['#', '0'..'9'] do inc(Run);
               TokenLen := Run - TokenPtr;
               SetString(TokenStr, TokenPtr, TokenLen);
               SetHTML;
               WriteToBuffer(Prefix + TokenStr + Postfix);
               TokenPtr := Run;
            end;

            '$':
            begin
               FTokenState := tsNumber;
               while Run^ in ['$', '0'..'9', 'A'..'F', 'a'..'f'] do inc(Run);
               TokenLen := Run - TokenPtr;
               SetString(TokenStr, TokenPtr, TokenLen);
               SetHTML;
               WriteToBuffer(Prefix + TokenStr + Postfix);
               TokenPtr := Run;
            end;

            else
            begin
               if Run^ <> #0 then
               begin
                  inc(Run);
                  TokenLen := Run - TokenPtr;
                  SetString(TokenStr, TokenPtr, TokenLen);
                  SetHTML;
                  WriteToBuffer(Prefix + TokenStr + Postfix);
                  TokenPtr := Run;
               end
               else
                  break;
            end;
         end;
      end;
      WriteToBuffer('</CODE>' + WRAP);
      WriteToBuffer('</PRE>' + WRAP);
      WriteToBuffer('<HR>' + WRAP);
      WriteToBuffer('<A NAME="bottom"></A>' + WRAP);
      WriteToBuffer('<CENTER><A HREF="#top">Jump to the top</A></CENTER>' + WRAP);
      WriteToBuffer('</BODY>' + WRAP);
      WriteToBuffer('</HTML>' + WRAP);
   end;
   Clear;
   SetPointer(FOutBuff, fBuffPos - 1);
end;  { ConvertReadStream }

procedure TPasConversion.Init;
begin
end;  { Initialize }

end.
